module WktimeHelper 
  include Redmine::Export::PDF

	def options_for_period_select(value)
		options_for_select([[l(:label_all_time), 'all'],
							[l(:label_this_week), 'current_week'],
							[l(:label_last_week), 'last_week'],
							[l(:label_this_month), 'current_month'],
							[l(:label_last_month), 'last_month'],
							[l(:label_this_year), 'current_year']],
							value.blank? ? 'current_month' : value)
	end
	
	# Indentation of Subprojects based on levels
	def options_for_wktime_project(projects, needBlankRow=false)
		projArr = Array.new
		if needBlankRow
			projArr << [ "", ""]
		end
		
		Project.project_tree(projects) do |proj_name, level|
			indent_level = (level > 0 ? ('&nbsp;' * 2 * level + '&#187; ').html_safe : '')
			sel_project = projects.select{ |proj| proj.name.to_s == proj_name.to_s }
			projArr << [ (indent_level + sel_project[0].name), sel_project[0].id ]
		end
		projArr
	end



  # Returns a PDF string of a weekly timesheet
  def wktime_to_pdf(entries, user, mon)

	# Landscape A4 = 210 x 297 mm
	page_height   = Setting.plugin_redmine_wktime['wktime_page_height'].to_i
	page_width    = Setting.plugin_redmine_wktime['wktime_page_width'].to_i
	right_margin  = Setting.plugin_redmine_wktime['wktime_margin_right'].to_i
	left_margin  = Setting.plugin_redmine_wktime['wktime_margin_left'].to_i
	bottom_margin = Setting.plugin_redmine_wktime['wktime_margin_bottom'].to_i
	top_margin = Setting.plugin_redmine_wktime['wktime_margin_top'].to_i
	col_id_width  = 10
	row_height    = Setting.plugin_redmine_wktime['wktime_line_space'].to_i
	logo    = Setting.plugin_redmine_wktime['wktime_header_logo']
	
	if page_height == 0
		page_height = 297
	end
	if page_width == 0
		page_width  = 210
	end	
	if right_margin == 0
		right_margin = 10
	end	
	if left_margin == 0
		left_margin = 10
	end
	if bottom_margin == 0
		bottom_margin = 20
	end
	if top_margin == 0
		top_margin = 20
	end
	if row_height == 0
		row_height = 4
	end
	
	# column widths
	table_width = page_width - right_margin - left_margin
	
	columns = [l(:field_project), l(:field_issue), l(:field_activity)]

	col_width = []
	orientation = "P"
	# 20% for project, 60% for issue, 20% for activity
	col_width[0] = (table_width - (8*10))*0.2
	col_width[1] = (table_width - (8*10))*0.6
	col_width[2] = (table_width - (8*10))*0.2

	for i in 0..6
		columns[3+i] = l('date.abbr_day_names')[(i+1)%7] + "\n" + (mon+i).mon().to_s() + "/" + (mon+i).day().to_s()
		col_width[3+i] = col_id_width
	end	
	
	#Landscape / Potrait
	if(table_width > 220)
		orientation = "L"
	else
		orientation = "P"
	end
	
	pdf = ITCPDF.new(current_language)
		
	pdf.SetTitle(l(:label_wktime))
	pdf.alias_nb_pages
	pdf.footer_date = format_date(Date.today)
	pdf.SetAutoPageBreak(false)
	pdf.AddPage(orientation)
	
	if !logo.blank?
		pdf.Image(Redmine::Plugin.public_directory + "/redmine_wktime/images/" + logo, page_width-10-20, 10)
	end
	
	render_header(pdf, entries, user, mon, row_height)

	pdf.Ln
	render_table_header(pdf, columns, col_width, row_height, col_id_width, table_width)

	weeklyHash = getWeeklyView(entries, true)
	col_values = []
	totals = [0.0,0.0,0.0,0.0,0.0,0.0,0.0]
	grand_total = 0.0
	j = 0
	base_x = pdf.GetX
	base_y = pdf.GetY
	max_height = row_height
	  
	weeklyHash.each do |key, matrix|
		unless matrix.blank?
			matrix.each do |rows|
				issueWritten = false
				col_values = []
				rows.each.with_index do |entry, i|
					unless entry.blank?
						if !issueWritten
							col_values[0] = entry.project.name
							col_values[1] = entry.issue.blank? ? "" : entry.issue.subject
							col_values[2] = entry.activity.name
							issueWritten = true
							j += 1
						end
						col_values[3+i] = entry.hours.blank? ? "" : entry.hours.to_s
						totals[i] += entry.hours unless entry.hours.blank?
						grand_total += entry.hours unless entry.hours.blank?
					end
				end
			end
		end

	  base_x = pdf.GetX
	  base_y = pdf.GetY
	  pdf.SetY(2 * page_height)
	  #write once to get the height
	  max_height = wktime_to_pdf_write_cells(pdf, col_values, col_width, row_height)
	  #reset the x and y
	  pdf.SetXY(base_x, base_y)

	  # make new page if it doesn't fit on the current one
	  space_left = page_height - base_y - bottom_margin
	  if max_height > space_left
		pdf.AddPage(orientation)
		pdf.Image(Redmine::Plugin.public_directory + "/redmine_wktime/images/" + logo, page_width-10-20, 10)
		render_table_header(pdf, columns, col_width, row_height, col_id_width, table_width)
		base_x = pdf.GetX
		base_y = pdf.GetY
	  end

	  # write the cells on page
	  pdf.RDMCell(col_id_width, row_height, j.to_s, "T", 0, 'C', 1)
	  #on the second time, do the actual write
	  wktime_to_pdf_write_cells(pdf, col_values, col_width, row_height)
	  issues_to_pdf_draw_borders(pdf, base_x, base_y, base_y + max_height, col_id_width, col_width)
	  


	  pdf.SetY(base_y + max_height);
	end
	#project, issue, is blank, and then total
	total_values = ["","","#{l(:label_total)} = #{grand_total}"]
	#concatenate two strings
	total_values += totals.collect{ |t| t.to_s}
	
	#write total
	#write an empty id
	pdf.RDMCell(col_id_width, row_height, "", "T", 0, 'C', 1)
	max_height = wktime_to_pdf_write_cells(pdf, total_values, col_width, row_height)
	pdf.SetY(base_y + max_height);

	render_signature(pdf, page_width, row_height)
	pdf.Output
  end
  
	# Renders MultiCells and returns the maximum height used
	def wktime_to_pdf_write_cells(pdf, col_values, col_widths,
								row_height)
		base_y = pdf.GetY
		max_height = row_height
		col_values.each_with_index do |val, i|
		  col_x = pdf.GetX

			pdf.RDMMultiCell(col_widths[i], row_height, val, "T", 'L', 1)
		  max_height = (pdf.GetY - base_y) if (pdf.GetY - base_y) > max_height
		  pdf.SetXY(col_x + col_widths[i], base_y);
		end
		return max_height
	end
	
	def getWeeklyView(entries, sumHours = false)
		weeklyHash = Hash.new
		prev_entry = nil
		entries.each do |entry|
			key = entry.project.id.to_s + (entry.issue.blank? ? '' : entry.issue.id.to_s) + entry.activity.id.to_s
			hourMatrix = weeklyHash[key]
			if hourMatrix.blank?
				#create a new matrix if not found
				hourMatrix =  []
				rows = []
				hourMatrix[0] = rows
				weeklyHash[key] = hourMatrix
			end
			
			#cwday returns 1 - 7, 1 is monday
			index = entry.spent_on.cwday - 1
			updated = false
			hourMatrix.each do |rows|
				if rows[index].blank?
					rows[index] = entry
					updated = true
					break
				else 
					if sumHours
						tempEntry = rows[index]
						tempEntry.hours += entry.hours
						updated = true
						break
					end
				end
			end
			if !updated
				rows = []
				hourMatrix[hourMatrix.size] = rows
				rows[index] = entry
			end

		end
		return weeklyHash
	end
	
	def render_table_header(pdf, columns, col_width, row_height, col_id_width, table_width)
        # headers
        pdf.SetFontStyle('B',8)
        pdf.SetFillColor(230, 230, 230)

        # render it background to find the max height used
        base_x = pdf.GetX
        base_y = pdf.GetY
        max_height = wktime_to_pdf_write_cells(pdf, columns, col_width, row_height)
        #pdf.Rect(base_x, base_y, table_width + col_id_width, max_height, 'FD');
		pdf.Rect(base_x, base_y, table_width, max_height, 'FD');
        pdf.SetXY(base_x, base_y);

        # write the cells on page
        pdf.RDMCell(col_id_width, row_height, "#", "T", 0, 'C', 1)
        wktime_to_pdf_write_cells(pdf, columns, col_width, row_height)
        issues_to_pdf_draw_borders(pdf, base_x, base_y, base_y + max_height, col_id_width, col_width)
        pdf.SetY(base_y + max_height);

        # rows
        pdf.SetFontStyle('',8)
        pdf.SetFillColor(255, 255, 255)
    end

	def render_header(pdf, entries, user, mon, row_height)
		base_x = pdf.GetX
		base_y = pdf.GetY
		  
		# title
		pdf.SetFontStyle('B',11)
		pdf.RDMCell(100,10, l(:label_wktime))
		pdf.SetXY(base_x, pdf.GetY+row_height)
		
		render_header_elements(pdf, base_x, pdf.GetY+row_height, l(:field_name), user.name)
		render_header_elements(pdf, base_x, pdf.GetY+row_height, l(:field_project), 
				entries.blank? ? "" : entries[0].project.name)
		render_header_elements(pdf, base_x, pdf.GetY+row_height, l(:label_wk_purchase_order))
		render_header_elements(pdf, base_x, pdf.GetY+row_height, l(:label_week), 
			mon.to_s + " - " + (mon+6).to_s)
	end
	
	def render_header_elements(pdf, x, y, element, value="")

		pdf.SetFontStyle('B',8)
		pdf.SetXY(x, y)
		pdf.RDMCell(50,10, element)
		pdf.SetXY(x+40, y)
		pdf.RDMCell(10,10, ":")
		pdf.SetFontStyle('',8)
		pdf.SetXY(x+40+2, y)
		pdf.RDMCell(50,10, value)

	end
	
	def render_signature(pdf, page_width, row_height)
		base_x = pdf.GetX
		base_y = pdf.GetY
		
		pdf.SetFontStyle('B',8)
		pdf.SetXY(page_width-90, pdf.GetY+row_height)
		pdf.RDMCell(50,10, l(:label_wk_signature) + " :")
		pdf.SetXY(page_width-90, pdf.GetY+(2*row_height))
		pdf.RDMCell(100,10, l(:label_wk_submitted_by) + " ________________________________")
		pdf.SetXY(page_width-90, pdf.GetY+ (2*row_height))
		pdf.RDMCell(100,10, l(:label_wk_approved_by) + " ________________________________")
	end
end